//+------------------------------------------------------------------+
//|                                                  Hull Trend.mq4  |
//|                                    Copyright  2005, adoleh2000  |
//|                                            adoleh2000@yahoo.com  |
//+------------------------------------------------------------------+
#property  copyright "Copyright  2005, adoleh2000."
#property  link      "adoleh2000@yahoo.com"
//---- indicator settings
#property  indicator_chart_window
#property  indicator_buffers 4
#property  indicator_color1  Blue
#property  indicator_color2  Red
#property  indicator_color3  DeepSkyBlue
#property  indicator_color4  MediumVioletRed
#property  indicator_width1  2
#property  indicator_width2  2

//---- indicator parameters
extern int    HMA_Period = 20;
extern double Price = PRICE_CLOSE;
extern double Displacement = 0;

//
//
//
//
//

extern bool showArrows      = false;
extern bool alertsOn        = false;
extern bool alertsOnCurrent = true;
extern bool alertsMessage   = true;
extern bool alertsSound     = false;
extern bool alertsEmail     = false;

//---- indicator buffers
double ind_buffer0[];
double ind_buffer1[];
double HighBuffer[];
double LowBuffer[];
double trend[];
int  draw_begin0;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
//---- indicator buffers mapping
   IndicatorBuffers(5);
   SetIndexBuffer(0, HighBuffer);
   SetIndexBuffer(1, LowBuffer);
      if (showArrows)
      {
         SetIndexStyle(0, DRAW_ARROW); SetIndexArrow(0,241);
         SetIndexStyle(1, DRAW_ARROW); SetIndexArrow(1,242);
      }
      else
      {
         SetIndexStyle(0, DRAW_HISTOGRAM);
         SetIndexStyle(1, DRAW_HISTOGRAM);
      }
   SetIndexBuffer(2, ind_buffer0);
   SetIndexBuffer(3, ind_buffer1);
   SetIndexBuffer(4, trend);
   draw_begin0 = HMA_Period + MathFloor(MathSqrt(HMA_Period));
   SetIndexDrawBegin(0, draw_begin0);
   SetIndexDrawBegin(1, draw_begin0);
   SetIndexDrawBegin(2, draw_begin0);
   SetIndexDrawBegin(3, draw_begin0);
   IndicatorDigits(Digits+1);
//---- name for DataWindow and indicator subwindow label
   IndicatorShortName("HT(" + HMA_Period + ")");
   SetIndexLabel(0, "HT(" + HMA_Period + ")");
   SetIndexLabel(1, "HT2");   
   SetIndexLabel(2, "HTHigh");
   SetIndexLabel(3, "HTLow");     
//---- initialization done
   return(0);
  }
//+------------------------------------------------------------------+
//| Hull Trend                                                       |
//+------------------------------------------------------------------+
int start()
  {
   int limit, i, shift;
   int counted_bars = IndicatorCounted();
//---- check for possible errors
   if(counted_bars < 1)
     {
       for(i = 1; i <= draw_begin0; i++) 
           ind_buffer0[Bars-i] = 0;
       for(i = 1; i <= HMA_Period; i++) 
           ind_buffer1[Bars-i] = 0;
     }
//---- last counted bar will be recounted
   if(counted_bars > 0) 
       counted_bars--;
   limit = Bars - counted_bars;
//---- MA difference counted in the 1-st buffer
   for(i = 0; i < limit; i++)
       ind_buffer1[i] = iMA(NULL, 0, MathFloor(HMA_Period / 2), 0, MODE_LWMA, Price, i)*2 - 
                        iMA(NULL, 0, HMA_Period, 0, MODE_LWMA, Price, i);
//---- HMA counted in the 0-th buffer
   for(i = 0; i < limit; i++)
       ind_buffer0[i] = iMAOnArray(ind_buffer1, 0, MathFloor(MathSqrt(HMA_Period)), 0, 
                        MODE_LWMA, i + Displacement);
//----
   for(shift = i; shift >= 0; shift--)
     {
      trend[shift] = trend[shift+1];
       if(ind_buffer1[shift] > ind_buffer0[shift])
         {
            trend[shift] = 1;
            if (!showArrows)
            {
               HighBuffer[shift] = High[shift];
               LowBuffer[shift] = Low[shift];
            }               
         }
       else
           if(ind_buffer1[shift] < ind_buffer0[shift])
             {
               trend[shift] = -1;
               if (!showArrows)
               {
                  LowBuffer[shift] = High[shift];
                  HighBuffer[shift] = Low[shift];
               }                  
             }
         if (showArrows)
         {
            HighBuffer[shift] = EMPTY_VALUE;
            LowBuffer[shift]  = EMPTY_VALUE;
            if (trend[shift]!=trend[shift+1])
               if (trend[shift]==1)
                     HighBuffer[shift] = ind_buffer1[shift]-iATR(NULL,0,10,shift)/2.0;
               else  LowBuffer[shift]  = ind_buffer1[shift]+iATR(NULL,0,10,shift)/2.0;
         }                         
     }

   //
   //
   //
   //
   //
        
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1;
      if (trend[whichBar] != trend[whichBar+1])
      if (trend[whichBar] == 1)
            doAlert("up");
      else  doAlert("down");       
   }
     
//---- done
   return(0); 
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+  
//
//
//
//
//

void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[0]) {
          previousAlert  = doWhat;
          previousTime   = Time[0];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," hull trend changed to ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol(),"hull trend  "),message);
             if (alertsSound)   PlaySound("alert2.wav");
      }
}

